using System;
using IGExtensions.Common.Maps.Assets.Resources;
using IGExtensions.Framework;
using Infragistics.Controls.Maps;

namespace IGExtensions.Common.Maps.Imagery // Infragistics.Controls.Maps 
{
    #region CloudMade Imagery View
    /// <summary>
    /// Represents a map view for the CloudMade geo-imagery. 
    /// </summary>
    public class CloudMadeMapImageryView : GeoImageryViewModel
    {
        public CloudMadeMapImageryView()
            : this(CloudMadeMapImageryStyle.FineLineMapStyle)
        { }
        public CloudMadeMapImageryView(CloudMadeMapImageryStyle imageryStyle)
        {
            this.ImageryKey = GeoMapImagryProvider.CloudMadeMapImageryConfig.ImageryKey;
            this.ImagerySource = GeoImagerySource.CloudMadeMapImagery;
            this.ImageryStyle = imageryStyle;
            this.ImageryDisplayName = ImageryStrings.Imagery_CloudMadeMap;
            this.ImageryDisplayName += " (" + imageryStyle.ToString().Remove("MapStyle") + ")";
        }

        #region Properties
        //TODO add support for custom style
        private CloudMadeMapImageryStyle _imageryStyle;
        public CloudMadeMapImageryStyle ImageryStyle
        {
            get { return _imageryStyle; }
            set { _imageryStyle = value; this.ImageryParameter = ((int)value).ToString(); OnPropertyChanged("ImageryStyle"); }
        }

        /// <summary>
        /// Gets or sets imagery parameter (style) for the CloudMade geo-imagery. 
        /// <para>Refer to http://maps.cloudmade.com/editor for complete list of available parameters</para>
        /// </summary>
        public string ImageryParameter
        {
            get { return _imageryParameter; }
            set { if (_imageryParameter == value) return; _imageryParameter = value; OnPropertyChanged("ImageryParameter"); }
        }
        private string _imageryParameter;
      
        /// <summary>
        /// Gets or sets imagery key for the CloudMade geo-imagery. 
        /// <para>You must get your own key from http://maps.cloudmade.com </para>
        /// </summary>
        public string ImageryKey
        {
            get { return _imageryKey; }
            set { if (_imageryKey == value) return; _imageryKey = value; OnPropertyChanged("ImageryKey"); }
        }
        private string _imageryKey;

        /// <summary>
        /// Gets the style name of the CloudMade geo-imagery. 
        /// </summary>
        public override string ImageryName
        {
            get
            {
                return this.ImagerySource.ToString().Replace("MapImagery", "") +
                       this.ImageryStyle.ToString().Replace("MapStyle", "Imagery");
            }
        }
        /// <summary>
        /// Gets the file name of the CloudMade geo-imagery. 
        /// </summary>
        public override string ImageryFileName { get { return this.ImageryName + ".png"; }}
        #endregion
        #region Methods
      
        public bool IsImageryKeyValid()
        {
            return !this.ImageryKey.Equals(string.Empty);
        }
        public CloudMadeMapImageryView FromParameter(int parameter)
        {
            var style = CloudMadeMapImageryStyle.FineLineMapStyle;
            try
            {
               // style = (CloudMadeMapImageryStyle)Enum.Parse(typeof(CloudMadeMapImageryStyle), parameter, true);
                var styleOb = Enum.ToObject(typeof(CloudMadeMapImageryStyle), parameter);

            }

            catch (Exception)
            {
                DebugManager.LogWarning("Failed to parse parameter for CloudMadeMapImagery: " + parameter);
                style = CloudMadeMapImageryStyle.FineLineMapStyle;
            }
            return new CloudMadeMapImageryView(style);
        }
        #endregion
    }
    #endregion
    public static class CloudMadeMapImageryViews
    {
        public static CloudMadeMapImageryView CloudMadeMapFineLineImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.FineLineMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapTouristImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.TouristMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapRedAlertImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.RedAlertMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapFreshImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.FreshMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapDarkGrayImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.DarkGrayMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapLightGrayImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.LightGrayMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapLightBlueImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.LightBlueMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapMidnightImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.MidnightMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapMarcoPoloImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.MarcoPoloMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapNederlandImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.NederlandMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapDarkVoyagerImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.DarkVoyagerMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapBlueLandImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.BlueLandMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapNightTimeImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.NightTimeMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapDarkMonoImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.DarkMonoMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapGoldenNightImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.GoldenNightMapStyle); }
        }
        public static CloudMadeMapImageryView CloudMadeMapDeepBlueImageryView
        {
            get { return new CloudMadeMapImageryView(CloudMadeMapImageryStyle.DeepBlueMapStyle); }
        } 
    }
  
      
}